@@ -1396,6 +1396,12 @@ terminal_app_init (TerminalApp *app)
   gconf_client_add_dir (app->conf, MONOSPACE_FONT_DIR,
                         GCONF_CLIENT_PRELOAD_ONELEVEL,
                         NULL);
+  gconf_client_add_dir (app->conf, CONF_PROXY_PREFIX,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL,
+                        NULL);
+  gconf_client_add_dir (app->conf, CONF_HTTP_PROXY_PREFIX,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL,
+                        NULL);
 
   app->profile_list_notify_id =
     gconf_client_notify_add (app->conf, PROFILE_LIST_KEY,
